#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phoebe_build_config.h"
#include "phoebe_global.h"

#include "phoebe_base.h"
#include "phoebe_error_handling.h"
#include "phoebe_scripter_ast.h"
#include "phoebe_scripter_core.h"
#include "phoebe_scripter_grammar.tab.h"

int main (int argc, char *argv[])
	{
	phoebe_debug ("starting up PHOEBE:\n");

	phoebe_debug ("  setting version number and version date.\n");
	PHOEBE_VERSION_NUMBER = strdup (PHOEBE_RELEASE_NAME);
	PHOEBE_VERSION_DATE = strdup (PHOEBE_RELEASE_DATE);

	phoebe_debug ("  initializing variables.\n");
	variables_init ();

	phoebe_debug ("  parsing the start-up line.\n");
	parse_startup_line (argc, argv);

	if (PHOEBE_args.INTERACTIVE_SCRIPTER_SWITCH == 1)
		{
		phoebe_debug ("  found a '-s' switch, proceeding to the scripter mode.\n");
		phoebe_debug ("  initializing internal PHOEBE structures.\n");
		phoebe_init ();

		phoebe_debug ("  initializing a symbol table.\n");
		scripter_st_initialize ();

		phoebe_debug ("  initializing the scripter.\n");
		scripter_initialize ();

		phoebe_debug ("  entering the main scripter loop.\n");
		phoebe_debug ("PHOEBE start-up successful.\n");
		scripter_main_loop ();
		}
	else if (PHOEBE_args.SCRIPT_SWITCH == 1)
		{
		FILE *in = fopen (PHOEBE_args.SCRIPT_NAME, "r");

		phoebe_debug ("  found a '-e' switch, proceeding to the execute mode.\n");

		if (in == NULL)
			{
			printf ("\n");
			phoebe_fatal ("cannot open script %s, aborting.\n\n", PHOEBE_args.SCRIPT_NAME);
			}

		phoebe_debug ("  script to be executed: %s.\n", PHOEBE_args.SCRIPT_NAME);

		phoebe_debug ("  initializing internal PHOEBE structures.\n");
		phoebe_init ();

		phoebe_debug ("  initializing a symbol table.\n");
		scripter_st_initialize ();

		phoebe_debug ("  initializing the scripter.\n");
		scripter_initialize ();

		phoebe_debug ("PHOEBE start-up successful.\n");
		phoebe_debug ("executing the script from the stream.\n");
		scripter_execute_script_from_stream (in);
		fclose (in);
		}
	else if (PHOEBE_args.CONFIGURE_SWITCH == 1)
		{
		phoebe_debug ("  found a '-c' switch, proceeding to configuration mode.\n");
		phoebe_init ();
		printf ("\n");
		}
	else
		{
		phoebe_debug ("  no switches, starting up in the GUI mode.\n");

		printf ("\n%s\n\n", PHOEBE_RELEASE_NAME);
		printf ("     Thank you for your interest in PHOEBE! Please note that this is the\n");
		printf ("     pre-release and as such it is meant *exclusively* for testing.  The\n");
		printf ("     Graphical User Interface (GUI) is turned off in this version, since\n");
		printf ("     its build process is independent of PHOEBE core (the scripter).\n\n");
		printf ("     Please start PHOEBE scripter by passing the '-s' switch in the com-\n");
		printf ("     mand line!\n\n");
/*
		gtk_set_locale ();
		gtk_init (&argc, &argv);
		phoebe_gui_init ();
		gtk_widget_show (PHOEBE);
		gtk_main ();
*/
		}

	return 0;
	}
