#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>

#define GLADE_HOOKUP_OBJECT(component,widget,name) \
  g_object_set_data_full (G_OBJECT (component), name, \
    gtk_widget_ref (widget), (GDestroyNotify) gtk_widget_unref)

#define GLADE_HOOKUP_OBJECT_NO_REF(component,widget,name) \
  g_object_set_data (G_OBJECT (component), name, widget)

#include "phoebe_build_config.h"
#include "phoebe_keywords.h"

int phoebe_notice (const char *fmt, ...)
	{
	va_list ap;
	int r;

	printf ("PHOEBE notice: ");
	va_start (ap, fmt);
	r = vprintf (fmt, ap);
	va_end (ap);

	return r;
	}

int phoebe_warning (const char *fmt, ...)
	{
	va_list ap;
	int r;

	printf ("PHOEBE warning: ");
	va_start (ap, fmt);
	r = vprintf (fmt, ap);
	va_end (ap);

	return r;
	}

int phoebe_debug (const char *fmt, ...)
	{
	/* This function writes the message to stdout in case PHOEBE was compiled   */
	/* with --enable-debug switch, otherwise it just returns control to the ma- */
	/* in program.                                                              */

	va_list ap;
	int r = -1;

	#ifdef PHOEBE_DEBUG_SUPPORT
		printf ("PHOEBE debug: ");
		va_start (ap, fmt);
		r = vprintf (fmt, ap);
		va_end (ap);
	#endif

	return r;
	}

int phoebe_fatal (const char *fmt, ...)
	{
	va_list ap;

	printf ("PHOEBE fatal: ");
	va_start (ap, fmt);
	vprintf (fmt, ap);
	va_end (ap);

	exit (EXIT_FAILURE);
	}

int phoebe_scripter_output (const char *fmt, ...)
	{
	va_list ap;
	int r;

	if (phoebe_get_value_int ("scripter_verbosity_level") == 0) return 0;

	printf ("PHOEBE scripter: ");
	va_start (ap, fmt);
	r = vprintf (fmt, ap);
	va_end (ap);

	return r;
	}

void *phoebe_malloc (size_t size)
	{
	register void *value = malloc (size);
	if (value == 0) phoebe_fatal ("Virtual memory exhauseted.\n");
	return value;
	}

void *phoebe_realloc (void *ptr, size_t size)
	{
	/* This function reallocates the space with the check whether the memory    */
	/* was exhausted. If the size equals 0, realloc calls free() on ptr.        */

	register void *value = realloc (ptr, size);
	if ( (value == 0) && (size != 0) ) phoebe_fatal ("Virtual memory exhauseted.\n");
	return value;
	}
