#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "phoebe_build_config.h"
#include "../include/cfortran.h"

#include "phoebe_accessories.h"
#include "phoebe_allocations.h"
#include "phoebe_calculations.h"
#include "phoebe_error_handling.h"
#include "phoebe_fitting.h"
#include "phoebe_fortran_interface.h"
#include "phoebe_global.h"
#include "phoebe_keywords.h"
#include "phoebe_type_handling.h"

#include "phoebe_scripter_ast.h"

PROTOCCALLSFFUN5(VOID,WDDC,wddc,STRING,STRING,DOUBLEV,DOUBLEV,DOUBLEV)
#define WD_FIND_MINIMUM_WITH_DC(FATMCOF,FATMCOFPL,CORRECTION,STDDEV,CHISQ) CCALLSFFUN5(WDDC,wddc,STRING,STRING,DOUBLEV,DOUBLEV,DOUBLEV,FATMCOF,FATMCOFPL,CORRECTION,STDDEV,CHISQ)

/* At this point we should check whether GSL is available; if it isn't, then  */
/* we don't have much to compile here.                                        */

#ifdef HAVE_LIBGSL
#ifndef PHOEBE_GSL_DISABLED

#include <gsl/gsl_min.h>
#include <gsl/gsl_multimin.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_siman.h>

double chi2_cost_function (const gsl_vector *adjpars, void *params)
	{
	/* This function evaluates chi2 of the O-C difference and returns its value */
	/* to the minimizer. The first passed argument is a simple vector of all    */
	/* parameters that are set for adjustment. The second parameter is the cast */
	/* passed_parameters struct defined below. *pars points to the address of   */
	/* the WD_LCI_parameters struct defined in the main minimizer function,     */
	/* **pointers is an array of pointers to double that point to those ele-    */
	/* ments of *pars which are set for adjustment.                             */

	typedef struct passed_parameters
		{
		int to_be_adjusted;
		int lcno;
		int rvno;
		bool rv1;
		bool rv2;
		int CALCHLA;
		int CALCVGA;
		bool ASINI;
		int CC;
		int *indices;
		PHOEBE_data **observations;
		double *sigma;
		double *average;
		WD_LCI_parameters **pars;
		double ***pointers;
		} passed_parameters;

	int i, curve;
	PHOEBE_data *syn_data;
	double L1, L2;
	double min, max;

	double        chi2         = 0.0;
	int           lcno         = (*(passed_parameters *) params).lcno;
	int           rvno         = (*(passed_parameters *) params).rvno;
	bool          rv1present   = (*(passed_parameters *) params).rv1;
	bool          rv2present   = (*(passed_parameters *) params).rv2;
	int           tba          = (*(passed_parameters *) params).to_be_adjusted;
	int           CALCHLA      = (*(passed_parameters *) params).CALCHLA;
	int           CALCVGA      = (*(passed_parameters *) params).CALCVGA;
	bool          ASINI        = (*(passed_parameters *) params).ASINI;
	int           CC           = (*(passed_parameters *) params).CC;
	int          *indices      = (*(passed_parameters *) params).indices;
	PHOEBE_data **observations = (*(passed_parameters *) params).observations;
	double       *sigma        = (*(passed_parameters *) params).sigma;
	double       *average      = (*(passed_parameters *) params).average;
	double     ***pointers     = (*(passed_parameters *) params).pointers;
	WD_LCI_parameters **pars   = (*(passed_parameters *) params).pars;

	for (curve = 0; curve < lcno; curve++)
		{
		syn_data = phoebe_data_new ();
		phoebe_data_alloc (syn_data, observations[curve]->ptsno);
		for (i = 0; i < observations[curve]->ptsno; i++)
			syn_data->indep[i] = observations[curve]->indep[i];

		/* Take care of conditional constraining if it's used:                    */
		if (CC == 1)
			{
			double T1, T2, P0, L1, L2, M1, M2, q, a, R1, R2, O1, O2;

			for (i = 0; i < tba; i++)
				{
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_teff1") == 0)
					T1 = gsl_vector_get (adjpars, i);
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_teff2") == 0)
					T2 = gsl_vector_get (adjpars, i);
				}

			P0 = pars[curve]->PERIOD;
			calculate_main_sequence_parameters (T1, T2, P0, &L1, &L2, &M1, &M2, &q, &a, &R1, &R2, &O1, &O2);
			pars[curve]->RM   = q;
			pars[curve]->SMA  = a;
			pars[curve]->PHSV = O1;
			pars[curve]->PCSV = O2;
			pars[curve]->HLA  = L1;
			}
		if (ASINI == TRUE)
			{
			/* 'indices' is a cno-dimensional array of integers holding indices to  */
			/* parameter qualifiers set for adjustment. We use this to find SMA and */
			/* INCL.                                                                */

			double sma;
			double incl;
			double asini = phoebe_get_value_double ("phoebe_asini");

			for (i = 0; i < tba; i++)
				{
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_sma") == 0)
					{
					sma = gsl_vector_get (adjpars, i);
					if (asini > sma) gsl_vector_set ((gsl_vector *) adjpars, i, sma = asini);
					incl = 180.0/M_PI*asin (asini/sma);
					if (pars[curve]->INCL > 90.0) incl += 90.0;
					pars[curve]->INCL = incl;
					break;
					}
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_incl") == 0)
					{
					incl = gsl_vector_get (adjpars, i);
					sma = asini / sin (incl/180.0*M_PI);
					pars[curve]->SMA = sma;
					break;
					}
				}
			}

		for (i = 0; i < tba; i++) *pointers[curve][i] = gsl_vector_get (adjpars, i);
		create_lci_file ("lcin.active", *pars[curve]);
		call_wd_to_get_fluxes (syn_data);

		/* Finally, we compute (rather than minimize) the levels HLA:           	*/
		if (CALCHLA == 1)
			{
			double av1 = calculate_average (syn_data);
			double hla = pars[curve]->HLA;
			pars[curve]->HLA = hla * average[curve] / av1;
			for (i = 0; i < syn_data->ptsno; i++)
				syn_data->dep[i] *= average[curve] / av1;
			}

		/* Normalize the data for optimal NMS handling: the idea is to normali- */
		/* ze *both* synthetic and observational curves to 1 at *synthetic*     */
		/* quarter-phase rather than observational quarter-phase, which is ill- */
		/* posed and only brings problems how to estimate it.                   */

		calculate_data_min_max (*syn_data, &min, &max);
		chi2 += calculate_chi2 (*syn_data, *observations[curve], sigma[curve]*max, 3);
		phoebe_data_free (syn_data);
		}

	if (rvno != 0)
		{
		PHOEBE_data *rv1, *rv2;

		/* RVs are sorted in observations[] so that RV1 always preceeds RV2.      */
		if (rv1present)
			{
			rv1 = phoebe_data_new ();
			phoebe_data_alloc (rv1, observations[curve]->ptsno);
			for (i = 0; i < observations[curve]->ptsno; i++)
				rv1->indep[i] = observations[curve]->indep[i];
			for (i = 0; i < tba; i++)
				*pointers[curve][i] = gsl_vector_get (adjpars, i);
			create_lci_file ("lcin.active", *pars[curve]);
			call_wd_to_get_rv1 (rv1);
			}
		if (rv2present && rv1present)
			{
			rv2 = phoebe_data_new ();
			phoebe_data_alloc (rv2, observations[curve+1]->ptsno);
			for (i = 0; i < observations[curve+1]->ptsno; i++)
				rv2->indep[i] = observations[curve+1]->indep[i];
			for (i = 0; i < tba; i++)
				*pointers[curve+1][i] = gsl_vector_get (adjpars, i);
			create_lci_file ("lcin.active", *pars[curve+1]);
			call_wd_to_get_rv2 (rv2);
			}
		if (rv2present && !rv1present)
			{
			rv2 = phoebe_data_new ();
			phoebe_data_alloc (rv2, observations[curve]->ptsno);
			for (i = 0; i < observations[curve]->ptsno; i++)
				rv2->indep[i] = observations[curve]->indep[i];
			for (i = 0; i < tba; i++)
				*pointers[curve][i] = gsl_vector_get (adjpars, i);
			create_lci_file ("lcin.active", *pars[curve]);
			call_wd_to_get_rv2 (rv2);
			}

		if (CALCVGA == 1)
			{
			double vga;

			if ( rv1present && !rv2present)
				vga = calculate_vga (rv1, NULL, average[curve], observations[curve]->ptsno, 0.0, 0, pars[curve]->VGA);
			if (!rv1present &&  rv2present)
				vga = calculate_vga (NULL, rv2, 0.0, 0, average[curve], observations[curve]->ptsno, pars[curve]->VGA);
			if ( rv1present &&  rv2present)
				vga = calculate_vga (rv1, rv2, average[curve], observations[curve]->ptsno, average[curve+1], observations[curve+1]->ptsno, pars[curve]->VGA);

			for (i = 0; i < lcno + rvno; i++) pars[i]->VGA = vga;
			}

		if (ASINI == TRUE)
			{
			double sma;
			double incl;
			double asini = phoebe_get_value_double ("phoebe_asini");

			for (i = 0; i < tba; i++)
				{
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_sma") == 0)
					{
					sma = gsl_vector_get (adjpars, i);
					if (asini > sma) gsl_vector_set ((gsl_vector *) adjpars, i, sma = asini);
					incl = 180.0/M_PI*asin (asini/sma);
					if (pars[curve]->INCL > 90.0) incl += 90.0;
					pars[curve]->INCL = incl;
					break;
					}
				if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_incl") == 0)
					{
					incl = gsl_vector_get (adjpars, i);
					sma = asini / sin (incl/180.0*M_PI);
					pars[curve]->SMA = sma;
					break;
					}
				}
			}

		if (rv1present)
			chi2 += calculate_chi2 (*rv1, *observations[curve], sigma[curve-lcno], 3);
		if (rv2present && rv1present)
			chi2 += calculate_chi2 (*rv2, *observations[curve+1], sigma[curve-lcno+1], 3);
		if (rv2present && !rv1present)
			chi2 += calculate_chi2 (*rv2, *observations[curve], sigma[curve-lcno], 3);

		if (rv1present) phoebe_data_free (rv1);
		if (rv2present) phoebe_data_free (rv2);
		}

	return chi2;
	}

int find_minimum_with_nms (double accuracy, int iter_no, double *final_chi2)
	{
	/****************************************************************************/
	/* This is a GSL simplex function that we use for multi-D minimization. All */
	/* parameters are passed through an array of structs.                       */
	/*                                                                          */
	/* Return values:                                                           */
	/*                                                                          */
	/*  -9 .. independent variable enumeration readout failed.                  */
	/*  -8 .. dependent variable enumeration readout failed.                    */
	/*  -7 .. weighting variable enumeration readout failed.                    */
	/*  -2 .. experimental data cannot be read in, abort the execution.         */
	/*  -1 .. passed values are invalid, abort the execution.                   */
	/*   0 .. everything ok, simplex converged.                                 */
	/*   1 .. no curves selected for adjustment, nothing to be done.            */
	/*   2 .. no parameters selected for adjustment, nothing to be done.        */
	/*   3 .. simplex reached maximum number of iterations without convergence. */
	/****************************************************************************/

	double *initvals;           /* An array of initial parameter values.        */
	int    *indices;            /* An array of global table reference indices   */
	int     cno, lcno, rvno;    /* Curves (both LC and RV) number               */
	int     CALCHLA;            /* Should light levels be calculated or fitted  */
	int     CALCVGA;            /* Should gamma velocity be calculated          */
	bool    ASINI;              /* A switch whether a \sin i should be constant */
	int     CC;                 /* Conditional constraining (CC) switch         */
	int     to_be_adjusted;     /* The number of adjustables to be adjusted     */
	double ***pointers;         /* Array of pointers to the WD_LCI_parameters   */
	PHOEBE_data **observations; /* An array of observations                     */
	double *sigma;              /* Sigmas of observations                       */
	double *average;            /* Average values of observations for HLA/VGA   */

	int     curve;

  const gsl_multimin_fminimizer_type *T = gsl_multimin_fminimizer_nmsimplex;
  gsl_multimin_fminimizer *s = NULL;
  gsl_vector *step_size, *adjpars;
  gsl_multimin_function cost_function;

  size_t iter = 0;
	size_t i, j, k;
  int status;
  double size;

	struct passed_parameters
		{
		int to_be_adjusted;
		int lcno;
		int rvno;
		bool rv1;
		bool rv2;
		int CALCHLA;
		int CALCVGA;
		bool ASINI;
		int CC;
		int *indices;
		PHOEBE_data **observations;
		double *sigma;
		double *average;
		WD_LCI_parameters **pars;
		double ***pointers;
		};
	struct passed_parameters passed_pars;

	WD_LCI_parameters *params;

	phoebe_debug ("entering downhill simplex minimizer.\n");

	/* Count the available curves:                                              */
	lcno = phoebe_get_value_int ("phoebe_lcno");
	rvno = phoebe_get_value_int ("phoebe_rvno");
	 cno = lcno + rvno;
	phoebe_debug ("total number of curves (both LC and RV): %d\n", lcno + rvno);

	if (cno == 0) return 1;

	passed_pars.rv1 = FALSE; passed_pars.rv2 = FALSE;
	for (i = 0; i < rvno; i++)
		{
		if (get_input_dependent_variable (phoebe_get_value_list_string ("phoebe_rv_dep", i)) == INPUT_PRIMARY_RV) passed_pars.rv1 = TRUE;
		if (get_input_dependent_variable (phoebe_get_value_list_string ("phoebe_rv_dep", i)) == INPUT_SECONDARY_RV) passed_pars.rv2 = TRUE;
		}
	phoebe_debug ("RV1 curve is present: %d\n", passed_pars.rv1);
	phoebe_debug ("RV2 curve is present: %d\n", passed_pars.rv2);

	/* The following block reads out parameters marked for adjustment:          */
	to_be_adjusted = read_in_adjustable_parameters (&initvals, &indices);
	phoebe_debug ("total number of parameters to be adjusted: %d\n", to_be_adjusted);
	if (to_be_adjusted == 0) return 2;

	phoebe_debug ("Parameters set for adjustment:\n");

	for (i = 0; i < to_be_adjusted; i++)
		{
		phoebe_debug ("%2d. qualifier:     %s\n", i+1, PHOEBE_parameters[indices[i]].qualifier);
		phoebe_debug ("    index:         %d\n", indices[i]);
		phoebe_debug ("    initial value: %lf\n", initvals[i]);
		}

	adjpars   = gsl_vector_alloc (to_be_adjusted);
	step_size = gsl_vector_alloc (to_be_adjusted);

	pointers  = phoebe_malloc (cno * sizeof (*pointers));
	for (i = 0; i < cno; i++)
		pointers[i]    = phoebe_malloc (to_be_adjusted * sizeof (**pointers));

	passed_pars.pars = phoebe_malloc (cno * sizeof (*(passed_pars.pars)));
	params           = phoebe_malloc (cno * sizeof (*params));
	observations     = phoebe_malloc (cno * sizeof (*observations));
	sigma            = phoebe_malloc (cno * sizeof (*sigma));
	average          = phoebe_malloc (cno * sizeof (*average));

	/* First we must read in all data, so that the following segment may se-    */
	/* quentially assign pointers to it:                                        */

	for (curve = 0; curve < lcno; curve++)
		read_in_wd_lci_parameters (&params[curve], /*MPAGE=*/1, curve);
	for (curve = lcno; curve < cno; curve++)
		read_in_wd_lci_parameters (&params[curve], /*MPAGE=*/2, curve-lcno);

	CALCHLA = params[0].CALCHLA;
	CALCVGA = params[0].CALCVGA;
	CC      = params[0].CC;
	ASINI   = params[0].ASINI;

	if (CALCHLA == 1)
		phoebe_debug ("light levels (HLAs), if any, set to be calculated\n");
	else
		phoebe_debug ("light levels (HLAs), if any, set to be fitted\n");
	if (CALCVGA == 1)
		phoebe_debug ("gamma velocity (VGA), if applicable, set to be calculated\n");
	else
		phoebe_debug ("gamma velocity (VGA), if applicable, set to be fitted\n");
	if (ASINI == TRUE)
		phoebe_debug ("The value of a sin(i) = %lf is kept constant\n", phoebe_get_value_double ("phoebe_asini"));

	phoebe_debug ("conditional constraint used: %d\n", CC);

	/* Now comes the pointing part:                                             */

	for (curve = 0; curve < cno; curve++)
		{
		k = 0;
		for (i = 0; i < to_be_adjusted; i++)
			{
			gsl_vector_set (adjpars, i, initvals[i]);
			gsl_vector_set (step_size, i, PHOEBE_parameters[indices[i]].step);

			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_hjd0")    == 0) pointers[curve][i] = &(params[curve].HJD0);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_period")  == 0) pointers[curve][i] = &(params[curve].PERIOD);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_dpdt")    == 0) pointers[curve][i] = &(params[curve].DPDT);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_pshift")  == 0) pointers[curve][i] = &(params[curve].PSHIFT);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_sma")     == 0) pointers[curve][i] = &(params[curve].SMA);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_rm")      == 0) pointers[curve][i] = &(params[curve].RM);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_incl")    == 0) pointers[curve][i] = &(params[curve].INCL);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_vga")     == 0) pointers[curve][i] = &(params[curve].VGA);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_teff1")   == 0) pointers[curve][i] = &(params[curve].TAVH);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_teff2")   == 0) pointers[curve][i] = &(params[curve].TAVC);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_pot1")    == 0) pointers[curve][i] = &(params[curve].PHSV);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_pot2")    == 0) pointers[curve][i] = &(params[curve].PCSV);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_logg1")   == 0) pointers[curve][i] = &(params[curve].LOGG1);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_logg2")   == 0) pointers[curve][i] = &(params[curve].LOGG2);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_met1")    == 0) pointers[curve][i] = &(params[curve].MET1);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_met2")    == 0) pointers[curve][i] = &(params[curve].MET2);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_ecc")     == 0) pointers[curve][i] = &(params[curve].E);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_perr0")   == 0) pointers[curve][i] = &(params[curve].PERR0);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_dperdt")  == 0) pointers[curve][i] = &(params[curve].DPERDT);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_f1")      == 0) pointers[curve][i] = &(params[curve].F1);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_f2")      == 0) pointers[curve][i] = &(params[curve].F2);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_alb1")    == 0) pointers[curve][i] = &(params[curve].ALB1);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_alb2")    == 0) pointers[curve][i] = &(params[curve].ALB2);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_grb1")    == 0) pointers[curve][i] = &(params[curve].GR1);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_grb2")    == 0) pointers[curve][i] = &(params[curve].GR2);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_hla")     == 0) pointers[curve][i] = &(params[k++].HLA);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_cla")     == 0) pointers[curve][i] = &(params[k++].CLA);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_el3")     == 0) pointers[curve][i] = &(params[k++].EL3);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_opsf")    == 0) pointers[curve][i] = &(params[k++].OPSF);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_ld_lcx1") == 0) pointers[curve][i] = &(params[k++].X1A);
			if (strcmp (PHOEBE_parameters[indices[i]].qualifier, "phoebe_ld_lcx2") == 0) pointers[curve][i] = &(params[k++].X2A);

			if (k == lcno) k = 0;
			}

		passed_pars.pars[curve] = &params[curve];

		/* Initialize observational data:                                         */
		observations[curve] = phoebe_data_new ();

		/* Read the data in and do all exception checking rigorously:             */
		if (curve < lcno)
			{
			char *file = phoebe_get_value_list_string   ("phoebe_lc_filename", curve);
			int indep  = get_input_independent_variable ((char *) phoebe_get_value_list_string ("phoebe_lc_indep", curve));
			int   dep  = get_input_dependent_variable   ((char *) phoebe_get_value_list_string ("phoebe_lc_dep", curve));
			int weight = get_input_weight               ((char *) phoebe_get_value_list_string ("phoebe_lc_indweight", curve));

			if ( indep == -1) return -9;
			if (   dep == -1) return -8;
			if (weight == -1) return -7;

			status = read_in_experimental_data (file, observations[curve], indep, OUTPUT_PHASE, dep, OUTPUT_TOTAL_FLUX, weight, OUTPUT_STANDARD_WEIGHT);
			}
		if (curve == lcno)
			{
			char *file = phoebe_get_value_list_string   ("phoebe_rv_filename", curve-lcno);
			int indep  = get_input_independent_variable ((char *) phoebe_get_value_list_string ("phoebe_rv_indep", curve-lcno));
			int   dep  = get_input_dependent_variable   ((char *) phoebe_get_value_list_string ("phoebe_rv_dep", curve-lcno));
			int weight = get_input_weight               ((char *) phoebe_get_value_list_string ("phoebe_rv_indweight", curve-lcno));
			int index;

			if (indep  == -1) return -9;
			if (  dep  == -1) return -8;
			if (weight == -1) return -7;

			if (rvno == 2)
				{
				if (dep == INPUT_PRIMARY_RV)
					status = read_in_experimental_data (file, observations[curve], indep, OUTPUT_PHASE, dep, OUTPUT_PRIMARY_RV, weight, OUTPUT_STANDARD_WEIGHT);
				if (dep == INPUT_SECONDARY_RV)
					status = read_in_experimental_data (file, observations[curve+1], indep, OUTPUT_PHASE, dep, OUTPUT_SECONDARY_RV, weight, OUTPUT_STANDARD_WEIGHT);
				}
			else
				{
				if (dep == INPUT_PRIMARY_RV)
					status = read_in_experimental_data (file, observations[curve], indep, OUTPUT_PHASE, dep, OUTPUT_PRIMARY_RV, weight, OUTPUT_STANDARD_WEIGHT);
				if (dep == INPUT_SECONDARY_RV)
					status = read_in_experimental_data (file, observations[curve], indep, OUTPUT_PHASE, dep, OUTPUT_SECONDARY_RV, weight, OUTPUT_STANDARD_WEIGHT);
				}

			/* Transform read experimental data to 100km/s units:                   */
			for (i = 0; i < observations[curve]->ptsno; i++)
				observations[curve]->dep[i] /= 100.0;
			}
		if (curve == lcno+1)
			{
			char *file = phoebe_get_value_list_string   ("phoebe_rv_filename", curve-lcno);
			int indep  = get_input_independent_variable ((char *) phoebe_get_value_list_string ("phoebe_rv_indep", curve-lcno));
			int   dep  = get_input_dependent_variable   ((char *) phoebe_get_value_list_string ("phoebe_rv_dep", curve-lcno));
			int weight = get_input_weight               ((char *) phoebe_get_value_list_string ("phoebe_rv_indweight", curve-lcno));

			if (indep  == -1) return -9;
			if (  dep  == -1) return -8;
			if (weight == -1) return -7;

			if (dep == INPUT_PRIMARY_RV)
				status = read_in_experimental_data (file, observations[curve-1], indep, OUTPUT_PHASE, dep, OUTPUT_PRIMARY_RV, weight, OUTPUT_STANDARD_WEIGHT);
			if (dep == INPUT_SECONDARY_RV)
				status = read_in_experimental_data (file, observations[curve], indep, OUTPUT_PHASE, dep, OUTPUT_SECONDARY_RV, weight, OUTPUT_STANDARD_WEIGHT);

			/* Transform read experimental data to 100km/s units:                   */
			for (i = 0; i < observations[curve]->ptsno; i++)
				observations[curve]->dep[i] /= 100.0;
			}

		if (status < 0) return -2;

		/* Get standard deviations and average values for observed data:          */

		if (curve < lcno)
			{
			int dep  = get_input_dependent_variable ((char *) phoebe_get_value_list_string ("phoebe_lc_dep", curve));
			phoebe_get_parameter_value ("phoebe_lc_sigma", curve, &(sigma[curve]));

			if (dep == INPUT_MAGNITUDE)
			/* (\Delta j + j) / j = 10^{2/5 \Delta m}, j = 1 at quarter-phase, so:  */
				sigma[curve] = pow (10, 2./5.*sigma[curve]) - 1.0;
			}
		if (curve == lcno)
			{
			phoebe_get_parameter_value ("phoebe_rv_sigma", 0, &(sigma[curve]));
			sigma[curve] /= 100.0;
			}
		if (curve == lcno+1)
			{
			phoebe_get_parameter_value ("phoebe_rv_sigma", 1, &(sigma[curve]));
			sigma[curve] /= 100.0;
			}
		average[curve] = calculate_weighted_average (observations[curve]);
		}

	passed_pars.to_be_adjusted = to_be_adjusted;
	passed_pars.lcno           = lcno;
	passed_pars.rvno           = rvno;
	passed_pars.CALCHLA        = CALCHLA;
	passed_pars.CALCVGA        = CALCVGA;
	passed_pars.ASINI          = ASINI;
	passed_pars.CC             = CC;
	passed_pars.indices        = indices;
	passed_pars.pointers       = pointers;
	passed_pars.observations   = observations;
	passed_pars.sigma          = sigma;
	passed_pars.average        = average;

  /* Initialize method and iterate */
	cost_function.f      = &chi2_cost_function;
	cost_function.n      = to_be_adjusted;
	cost_function.params = (void *) &passed_pars;

	s = gsl_multimin_fminimizer_alloc (T, to_be_adjusted);
	gsl_multimin_fminimizer_set (s, &cost_function, adjpars, step_size);

	do
		{
		iter++;

		status = gsl_multimin_fminimizer_iterate (s);
      
		if (status) break;

		size = gsl_multimin_fminimizer_size (s);
		status = gsl_multimin_test_size (size, accuracy);

		if (status == GSL_SUCCESS)
			phoebe_scripter_output ("converged to minimum at\n");
		fprintf (PHOEBE_output, "%3d ", iter);
		for (i = 0; i < to_be_adjusted; i++)
			fprintf (PHOEBE_output, "%6.3lf ", gsl_vector_get (s->x, i));
		for (i = 0; i < lcno; i++)
			fprintf (PHOEBE_output, "L%d = %6.3f ", i+1, params[i].HLA);
		if (CC == 1)
			fprintf (PHOEBE_output, "T1 = %0.0f T2 = %0.0f q = %4.4f i = %4.4f a = %4.4f O1 = %4.4f O2 = %4.4f ", params[0].TAVH, params[0].TAVC, params[0].RM, params[0].INCL, params[0].SMA, params[0].PHSV, params[0].PCSV);
		if (ASINI == TRUE)
			fprintf (PHOEBE_output, "a = %4.4f i = %4.4f ", params[0].SMA, params[0].INCL);
		if (lcno != cno)               /* Print VGA only if there are RVs present */
			fprintf (PHOEBE_output, "VGA = %3.3f ", params[cno-1].VGA);
		fprintf (PHOEBE_output, "chi2 = %10.3f step = %.3f\n", s->fval, size);
		}
	while (status == GSL_CONTINUE && iter < iter_no);

	/* If the convergence wasn't reached in iter_no steps, gsl minimizer re-    */
	/* turns -2, which conflicts with PHOEBE's way of enumerating exceptions    */
	/* (positive values for non-critical exceptions and negative for fatal      */
	/* ones). Thus we change the exit status to the predefined value 3:         */
	if (status != GSL_SUCCESS && iter == iter_no)
		status = 3;
	phoebe_debug ("NMS exit status: %d\n", status);

	/* Now we update the values in the main table:                              */
	for (i = 0; i < to_be_adjusted; i++)
		{
		if (PHOEBE_parameters[indices[i]].type == TYPE_DOUBLE)
			{
			phoebe_set_parameter_value (PHOEBE_parameters[indices[i]].qualifier, gsl_vector_get (s->x, i));
			phoebe_debug ("setting %s to its converged value %lf\n", PHOEBE_parameters[indices[i]].qualifier, phoebe_get_value_double (PHOEBE_parameters[indices[i]].qualifier));
			}

		if (PHOEBE_parameters[indices[i]].type == TYPE_DOUBLE_ARRAY)
			{
			int idx;
			if (strcmp (PHOEBE_parameters[indices[i]].dependency, "phoebe_lcno") == 0) idx = lcno;
			if (strcmp (PHOEBE_parameters[indices[i]].dependency, "phoebe_rvno") == 0) idx = rvno;
			for (j = 0; j < idx; j++)
				{
				phoebe_debug ("setting %s[%d] to its converged value %lf\n", PHOEBE_parameters[indices[i]].qualifier, j, gsl_vector_get (s->x, i));
				phoebe_set_parameter_value (PHOEBE_parameters[indices[i]].qualifier, j, gsl_vector_get (s->x, i));
				i++;
				}
			}
		}

	/* If HLA's were calculated (rather than fitted), update their values:      */
	if (CALCHLA == 1)
		for (i = 0; i < lcno; i++)
			phoebe_set_parameter_value ("phoebe_hla", i, params[i].HLA);

	/* We do the same with v_\gamma:                                            */
	if (CALCVGA == 1 && rvno != 0)
		phoebe_set_parameter_value ("phoebe_vga", params[0].VGA);

	/* Finally, let's put the converged chi2 value to the passed variable:      */
	*final_chi2 = s->fval;

	gsl_multimin_fminimizer_free (s);

	gsl_vector_free (adjpars);
	gsl_vector_free (step_size);

	for (i = 0; i < cno; i++)
		{
		free (pointers[i]);
		phoebe_data_free (observations[i]);
		}

	free (initvals);
	free (indices);

	free (pointers);
	free (params);
	free (observations);

	phoebe_debug ("leaving downhill simplex minimizer.\n");

  return status;
	}

int kick_parameters (double sigma)
	{
	/* This function kicks current parameters stochastically, following the     */
	/* Gaussian PDF. The passed argument is the relative standard deviation,    */
	/* \sigma_{rel} = \sigma_{abs} / parvalue.                                  */

	int to_be_adjusted;
	double *initvals;           /* An array of initial parameter values.        */
	int    *indices;            /* An array of global table reference indices   */
	int lcno, rvno;
	int i;

	WD_LCI_parameters params;

	phoebe_debug ("entering kick_parameters ()\n");

	/* Count the available curves:                                              */
	lcno = phoebe_get_value_int ("phoebe_lcno");
	rvno = phoebe_get_value_int ("phoebe_rvno");

	/* The following block reads out parameters marked for adjustment:          */
	to_be_adjusted = read_in_adjustable_parameters (&initvals, &indices);

	for (i = 0; i < to_be_adjusted; i++)
		if (PHOEBE_parameters[indices[i]].type == TYPE_DOUBLE)
			{
			phoebe_set_parameter_value (PHOEBE_parameters[indices[i]].qualifier, initvals[i] + gsl_ran_gaussian (PHOEBE_randomizer, sigma * initvals[i]));
			phoebe_debug ("  kicked %s from %lf to %lf\n", PHOEBE_parameters[indices[i]].qualifier, initvals[i], PHOEBE_parameters[indices[i]].value.d);
			}
		else
#warning IMPLEMENT CURVE-DEPENDENT KICKS!
			phoebe_scripter_output ("cannot kick curve-dependent parameters yet, sorry.\n");


	free (initvals);
	free (indices);

	phoebe_debug ("leaving kick_parameters ()\n");
	return 0;
	}

#endif
#endif

int find_minimum_with_dc (double *chi2ret)
	{
	/****************************************************************************/
	/* This is WD's built-in DC algorithm and as such it doesn't depend on GSL. */
	/* A macro WD_FIND_MINIMUM_WITH_DC is provided to access the fortran sub-   */
	/* routine.                                                                 */
	/*                                                                          */
	/* Return values:                                                           */
	/*                                                                          */
	/*  -9 .. independent variable enumeration readout failed.                  */
	/*  -8 .. dependent variable enumeration readout failed.                    */
	/*  -7 .. weighting variable enumeration readout failed.                    */
	/*  -2 .. experimental data cannot be read in, abort the execution.         */
	/*  -1 .. passed values are invalid, abort the execution.                   */
	/*   0 .. everything ok, simplex converged.                                 */
	/*   1 .. no curves selected for adjustment, nothing to be done.            */
	/*   2 .. no parameters selected for adjustment, nothing to be done.        */
	/*   3 .. simplex reached maximum number of iterations without convergence. */
	/****************************************************************************/

	char *pars[] =
		{
		"phoebe_spots_lat1",
		"phoebe_spots_long1",
		"phoebe_spots_rad1",
		"phoebe_spots_temp1",
		"phoebe_spots_lat2",
		"phoebe_spots_long2",
		"phoebe_spots_rad2",
		"phoebe_spots_temp2",
		"phoebe_sma",
		"phoebe_ecc",
		"phoebe_perr0",
		"phoebe_f1",
		"phoebe_f2",
		"phoebe_pshift",
		"phoebe_vga",
		"phoebe_incl",
		"phoebe_grb1",
		"phoebe_grb2",
		"phoebe_teff1",
		"phoebe_teff2",
		"phoebe_alb1",
		"phoebe_alb2",
		"phoebe_pot1",
		"phoebe_pot2",
		"phoebe_rm",
		"phoebe_hjd0",
		"phoebe_period",
		"phoebe_dpdt",
		"phoebe_dperdt",
		"",
		"phoebe_hla",
		"phoebe_cla",
		"phoebe_ld_lcx1",
		"phoebe_ld_lcx2",
		"phoebe_el3"
		};

	WD_DCI_parameters *params = phoebe_malloc (sizeof (*params));
	int i, j, index = 0;

	char atmcof[255], atmcofplanck[255];

	/* Define arrays that will hold DC results; they will be allocated when a   */
	/* number of parameters set for adjustment are known.                       */
	double *corrections;
	double *errors;

	int marked_tba = read_in_wd_dci_parameters (params);

	if (marked_tba == -9) return -9; /* Invalid indep variable */
	if (marked_tba == -8) return -8; /* Invalid dep variable */
	if (marked_tba == -7) return -7; /* Invalid levweight variable */
	if (marked_tba == -6) return  1; /* No curves to fit */
	if (marked_tba == -5) return  2; /* No parameters to fit */
	if (marked_tba == -2) return -2; /* Invalid data files */

	sprintf (atmcof,       "%s/wd/atmcof.dat",       PHOEBE_BASE_DIR);
	sprintf (atmcofplanck, "%s/wd/atmcofplanck.dat", PHOEBE_BASE_DIR);

	corrections = phoebe_malloc (marked_tba * sizeof (*corrections));
	errors      = phoebe_malloc (marked_tba * sizeof (*errors));

	create_dci_file ("dcin.active", params);
	WD_FIND_MINIMUM_WITH_DC (atmcof, atmcofplanck, corrections, errors, chi2ret);

	fprintf (PHOEBE_output, "%-18s %-12s %-12s %-12s %-12s\n", "Qualifier:", "Original:", "Correction:", "   New:", "  Error:");
	fprintf (PHOEBE_output, "--------------------------------------------------------------------\n");
	for (i = 0; i < 35; i++)
		if (params->tba[i] == 0)
			{
			if (i > 29)
				{
				for (j = 0; j < params->nlc; j++)
					{
					double val = phoebe_get_value_list_double (pars[i], j);
					int idx;
					fprintf (PHOEBE_output, "%s[%d]", pars[i], j+1);
					for (idx = strlen(pars[i]); idx <= 12; idx++) fprintf (PHOEBE_output, " ");
					fprintf (PHOEBE_output, "%12.6lf %12.6lf %12.6lf %12.6lf\n", val, corrections[index], val + corrections[index], errors[index]);
					phoebe_set_parameter_value (pars[i], j, val + corrections[index]);
					index++;
					}
				}
			else
				{
				double val = phoebe_get_value_double (pars[i]);
				int idx;
				fprintf (PHOEBE_output, "%s", pars[i]);
				if (strcmp (pars[i],   "phoebe_vga") == 0) { corrections[index] *=  100.0; val *=  100.0; }
				if (strcmp (pars[i], "phoebe_teff1") == 0) { corrections[index] *= 1000.0; val *= 1000.0; }
				if (strcmp (pars[i], "phoebe_teff2") == 0) { corrections[index] *= 1000.0; val *= 1000.0; }
				for (idx = strlen(pars[i])-3; idx <= 12; idx++) fprintf (PHOEBE_output, " ");
				fprintf (PHOEBE_output, "%12.6lf %12.6lf %12.6lf %12.6lf\n", val, corrections[index], val + corrections[index], errors[index]);
				phoebe_set_parameter_value (pars[i], val + corrections[index]);
				index++;
				}
			}

	wd_dci_parameters_free (params);

	free (corrections);
	free (errors);

	return 0;
	}

#warning NMS_PRINT () NOT YET IMPLEMENTED
int nms_print (const char *fmt, ...)
	{
	va_list ap;
	int r;

	va_start (ap, fmt);
	r = vprintf (fmt, ap);
	va_end (ap);

	return r;
	}
