#ifndef PHOEBE_SCRIPTER_AST_H
	#define PHOEBE_SCRIPTER_AST_H 1

#include <stdio.h>

#include "phoebe_global.h"
#include "phoebe_scripter_grammar.h"

FILE *PHOEBE_output;

typedef enum
	{
	kind_add,
	kind_and,
	kind_block,
	kind_builtin,
	kind_calc,
	kind_clear,
	kind_column,
	kind_command,
	kind_data,
	kind_datastat,
	kind_dec,
	kind_decby,
	kind_define,
	kind_div,
	kind_divby,
	kind_element,
	kind_equal,
	kind_execute,
	kind_execute_macro,
	kind_export,
	kind_for,
	kind_func,
	kind_gequal,
	kind_get_parameter_value,
	kind_greater,
	kind_help,
	kind_idiv,
	kind_if,
	kind_ifelse,
	kind_ignore,
	kind_inc,
	kind_incby,
	kind_info,
	kind_lequal,
	kind_less,
	kind_minimize_using_dc,
	kind_minimize_using_nms,
	kind_mul,
	kind_multby,
	kind_nequal,
	kind_not,
	kind_or,
	kind_os_cd,
	kind_os_ls,
	kind_os_pwd,
	kind_pot,
	kind_print,
	kind_proc,
	kind_qual_value,
	kind_return,
	kind_set,
	kind_set_element,
	kind_set_vector,
	kind_show,
	kind_statement,
	kind_stdump,
	kind_sub,
	kind_system_call,
	kind_unarym,
	kind_unaryp,
	kind_while
	} scripter_ast_kind;

typedef struct scripter_ast
	{
  enum
		{
		ast_int,
		ast_double,
		ast_bool,
		ast_string,
		ast_element,
		ast_vector,
		ast_array,
		ast_data,
		ast_variable,
		ast_function,
		ast_node
		} type;
  union
		{
		int            integer;
		double         real;
		bool           boolean;
		char          *string;
		PHOEBE_vector *vec;
		PHOEBE_array  *array;
		PHOEBE_data   *data;
		char          *variable;
		char          *function;
		struct
			{
			scripter_ast_kind         kind;
			struct scripter_ast_list *args;
			}    node;
		} value;
	} scripter_ast;

typedef struct scripter_ast_list
	{ 
	scripter_ast *elem;
	struct scripter_ast_list *next;
	} scripter_ast_list;

typedef struct scripter_ast_value
	{
	enum
		{
		type_int,
		type_bool,
		type_double,
		type_string,
		type_vector,
		type_array,
		type_data,
		type_void,
		} type;
	anytype value;
	} scripter_ast_value;

scripter_ast      *scripter_ast_add_int          (const int val);
scripter_ast      *scripter_ast_add_double       (const double val);
scripter_ast      *scripter_ast_add_bool         (const bool val);
scripter_ast      *scripter_ast_add_string       (const char *val);
scripter_ast      *scripter_ast_add_array        (PHOEBE_array *array);
scripter_ast      *scripter_ast_add_vector       (PHOEBE_vector *vec);
scripter_ast      *scripter_ast_add_data         (PHOEBE_data *data);
scripter_ast      *scripter_ast_add_literal      (const char *val);
scripter_ast      *scripter_ast_add_variable     (const char *val);
scripter_ast      *scripter_ast_add_function     (const char *val);
scripter_ast      *scripter_ast_add_node         (const scripter_ast_kind kind, scripter_ast_list *args);

scripter_ast_list *scripter_ast_construct_list   (scripter_ast *ast, scripter_ast_list *list);
scripter_ast_list *scripter_ast_reverse_list     (scripter_ast_list *r, scripter_ast_list *s);
int                scripter_ast_list_length      (scripter_ast_list *in);
scripter_ast_value scripter_ast_evaluate         (scripter_ast *in);

int                scripter_scope_level;
int                scripter_flow_scope_level;

void               scripter_ast_flow_begin_scope ();
void               scripter_ast_flow_end_scope ();
void               scripter_ast_flow_export_to_main ();

void scripter_ast_print_list (scripter_ast_list *in);
void scripter_ast_print (scripter_ast *in);

bool               scripter_return_flag;
scripter_ast_value scripter_return_value;

/* **********  This part of the code describes the Symbol Table:  *********** */

typedef struct scripter_symbol
	{
	char                   *name;
	scripter_ast           *link;
	int                     depth;
	struct scripter_symbol *next;
	} scripter_symbol;

#define scripter_st_size   97
scripter_symbol *scripter_st[scripter_st_size];

void             scripter_st_initialize           ();
unsigned int     scripter_st_identifier_hash      (const char *id);
void             scripter_st_identifier_assign    (const char *id, const double value);
int              scripter_st_identifier_commit    (const char *id, scripter_ast *ast, int depth);
scripter_symbol *scripter_st_identifier_lookup    (const char *id);
void             scripter_st_print                ();

#endif
