#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phoebe_build_config.h"
#include "phoebe_error_handling.h"
#include "phoebe_scripter_directives.h"

#ifdef HAVE_LIBREADLINE
	#ifndef PHOEBE_READLINE_DISABLED
		#include <readline/readline.h>
		#include <readline/history.h>
	#endif
#endif

int scripter_main_loop ()
	{
	/* This function does the line buffering for lexer. All user interaction    */
	/* goes through here. The loop is endless and only quit directive (or a     */
	/* segmentation fault ;) ) breaks it.                                       */

	char prompt[5];
	char *buffer = NULL;
	char *line;

	int i, block_level;

	phoebe_debug ("main scripter loop entered.\n");

	block_level = 0;
	sprintf (prompt, "> ");

#ifdef HAVE_LIBREADLINE
#ifndef PHOEBE_READLINE_DISABLED
	phoebe_debug ("initializing GNU readline command line history.\n");
	using_history ();
#endif
#endif

	do
		{
#ifdef HAVE_LIBREADLINE
#ifndef PHOEBE_READLINE_DISABLED
		phoebe_debug ("initializing GNU readline prompt.\n");
		line = readline (prompt);
		if (line == '\0') printf ("\n"), scripter_directive_quit ();
		else if (strlen(line) > 0) add_history (line);
#endif
#endif

#ifdef HAVE_LIBREADLINE
#ifdef PHOEBE_READLINE_DISABLED
		phoebe_debug ("initializing built-in (primitive) prompt.\n");
		line = phoebe_malloc (255);
		printf ("%s", prompt);
		fgets (line, 255, stdin);
#endif
#endif

#ifndef HAVE_LIBREADLINE
		phoebe_debug ("initializing built-in (primitive) prompt.\n");
		line = phoebe_malloc (255);
		printf ("%s", prompt);
		fgets (line, 255, stdin);
#endif

		if (line != '\0')
			{
			for (i = 0; i < strlen(line); i++)
				{
				if (line[i] == '#') { line[i] = '\0'; break; }
				if (line[i] == '{') block_level++;
				if (line[i] == '}') block_level--;
				}
			buffer = malloc (strlen (line)+2);
			strcpy (buffer, line);
			strcat (buffer, " ");
			free (line);
			}

		while (block_level > 0)
			{
			sprintf (prompt, "%2d> ", block_level);

#ifdef HAVE_LIBREADLINE
#ifndef PHOEBE_READLINE_DISABLED
			line = readline (prompt);
			if (line == '\0') printf ("\n"), scripter_directive_quit ();
			else if (strlen(line) > 0) add_history (line);
#endif
#endif
#ifdef HAVE_LIBREADLINE
#ifdef PHOEBE_READLINE_DISABLED
			line = phoebe_malloc (255);
			printf ("%s", prompt);
			fgets (line, 255, stdin);
#endif
#endif
#ifndef HAVE_LIBREADLINE
			line = phoebe_malloc (255);
			printf ("%s", prompt);
			fgets (line, 255, stdin);
#endif

			for (i = 0; i < strlen(line); i++)
				{
				if (line[i] == '#') { line[i] = '\0'; break; }
				if (line[i] == '{') block_level++;
				if (line[i] == '}') block_level--;
				}
			buffer = realloc (buffer, strlen (buffer) + strlen (line) + 2);
			strcat (buffer, line);
			strcat (buffer, " ");
			free (line);
			}

		if (line != 0)
			{
			yy_scan_string (buffer);
			yyparse ();
			}
		free (buffer);
		sprintf (prompt, "> ");
		}
	while (1 != 2);
	return 0;
	}

int scripter_execute_script_from_stream (FILE *stream)
	{
	char *buffer = NULL;

	while (!feof (stream))
		{
		char line[255];
		int i;

		fgets (line, 255, stream);
		if (feof (stream)) break;
		for (i = 0; i < strlen(line); i++)
			if (line[i] == '#')
				{
				line[i] = '\0';
				break;
				}
		if (strlen(line) == 0) continue;
		if (buffer == NULL)
			{
			buffer = phoebe_malloc (strlen (line)+1);
			strcpy (buffer, line);
			}
		else
			{
			buffer = phoebe_realloc (buffer, strlen(buffer) + strlen(line) + 1);
			strcat (buffer, line);
			}
		}

	yy_scan_string (buffer);
	yyparse ();
	free (buffer);

	return 0;
	}

int scripter_execute_script_from_buffer (char *buffer)
	{
	/* This is a simple function that calls the lexer/parser on 'buffer'.       */

	yy_scan_string (buffer);
	yyparse ();
	return 0;
	}
