#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "phoebe_build_config.h"

#include "phoebe_accessories.h"
#include "phoebe_error_handling.h"
#include "phoebe_global.h"
#include "phoebe_scripter_core.h"

int intern_scripter_print_help (const char *synopsis, const char *summary, const char *description)
	{
	/* This function takes all fields and writes them to screen in a unified    */
	/* manner.                                                                  */

	printf ("\n");
	printf ("  Synopsis:\t%s\n", synopsis);
	printf ("  Summary:\t%s\n", summary);
	printf ("  Description:\t%s\n\n", description);

	return 0;
	}

int scripter_directive_help (char *command)
	{
	/* This function gives help about a particular command passed as argument.  */

	if (strlen (command) == 0)
		{
		/* This means generic help directive with no arguments passed.            */
		printf ("\n  This is %s scripter. For general help on PHOEBE\n", PHOEBE_VERSION_NUMBER);
		printf ("  scripting please refer to PHOEBE API documentation. For specific online help\n");
		printf ("  use 'help help' or 'help command'. To quit, use 'quit'.\n\n");
		}
	else if (strcmp (command, "help") == 0)
		intern_scripter_print_help ("help [command]", "PHOEBE scripter's online help on commands and directives", "Please refer to PHOEBE API for further assistance.");
	else
		{
		printf ("\n  Sorry, command or directive '%s' isn't recognized.\n", command);
		printf ("  To list available commands and directives, use 'list'.\n\n");
		}

	return 0;
	}

int scripter_directive_quit ()
	{
	int c;

	phoebe_scripter_output ("are you sure you want to quit [y/N]: ");
	c = getchar ();
	if ( ( c  == 'y' ) || (c == 'Y') )
		exit (0);
	
	return 0;
	}

int scripter_directive_execute (char *filename)
	{
	/* This directive opens an external script, reads it into the buffer and    */
	/* passes it to the lexer.                                                  */
	/*                                                                          */
	/* Return values:                                                           */
	/*   -1  ..  filename not found                                             */
	/*   -2  ..  filename cannot be read                                        */
	/*   -3  ..  filename is a directory                                        */
	/*   -4  ..  filename open failure                                          */

	FILE *script;

	if (!filename_exists (filename)) return -1;
	if (!filename_has_read_permissions (filename)) return -2;
	if (filename_is_directory (filename)) return -3;
	script = fopen (filename, "r"); if (script == NULL) return -4;
	scripter_execute_script_from_stream (script);
	fclose (script);

	return 0;
	}
