#ifndef PHOEBE_TYPE_HANDLING_H
	#define PHOEBE_TYPE_HANDLING_H

#include "phoebe_global.h"

#include "phoebe_scripter_ast.h"

PHOEBE_vector *phoebe_vector_new                ();
PHOEBE_vector *phoebe_vector_new_from_qualifier (char *qualifier);
PHOEBE_vector *phoebe_vector_new_from_list      (scripter_ast_list *list);
PHOEBE_vector *phoebe_vector_new_from_column    (char *filename, int col);
PHOEBE_vector *phoebe_vector_duplicate          (PHOEBE_vector *vec);
int            phoebe_vector_alloc              (PHOEBE_vector *vec, int dimension);
int            phoebe_vector_realloc            (PHOEBE_vector *vec, int dimension);
int            phoebe_vector_pad                (PHOEBE_vector *vec, double value);
int            phoebe_vector_print              (PHOEBE_vector *vec);
int            phoebe_vector_free               (PHOEBE_vector *vec);

int            phoebe_vector_add                (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_subtract           (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_multiply           (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_divide             (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_raise              (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_multiply_by        (PHOEBE_vector *fac1, double factor);
int            phoebe_vector_dot_product        (double *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_vec_product        (PHOEBE_vector *result, PHOEBE_vector *fac1, PHOEBE_vector *fac2);
int            phoebe_vector_submit             (PHOEBE_vector *result, PHOEBE_vector *vec, double func ());
int            phoebe_vector_norm               (double *result, PHOEBE_vector *vec);
int            phoebe_vector_dim                (int *result, PHOEBE_vector *vec);
int            phoebe_vector_randomize          (PHOEBE_vector *result, double limit);
bool           phoebe_vector_compare            (PHOEBE_vector *vec1, PHOEBE_vector *vec2);

PHOEBE_data   *phoebe_data_new                  ();
PHOEBE_data   *phoebe_data_duplicate            (PHOEBE_data *data);
int            phoebe_data_alloc                (PHOEBE_data *data, int dim);
int            phoebe_data_realloc              (PHOEBE_data *data, int dim);
int            phoebe_data_uniform              (PHOEBE_data *data, double start, double end, int vertices);
int            phoebe_data_free                 (PHOEBE_data *data);

int            phoebe_data_multiply_by          (PHOEBE_data *result, PHOEBE_data *fac1, double factor);

int            phoebe_lc_params_update          (WD_LCI_parameters params, int curve);
int            phoebe_rv_params_update          (WD_LCI_parameters params, int curve);

PHOEBE_array  *phoebe_array_new                 (PHOEBE_type type);
int            phoebe_array_alloc               (PHOEBE_array *array, int dimension);
PHOEBE_array  *phoebe_array_new_from_qualifier  (char *qualifier);
PHOEBE_array  *phoebe_array_duplicate           (PHOEBE_array *array);
int            phoebe_array_print               (PHOEBE_array *array);
int            phoebe_array_free                (PHOEBE_array *array);

#endif
